from comon import *













import traceback
from collections import defaultdict

import pandas as pd
from flask import Flask, jsonify, request
from flask_cors import CORS, cross_origin
from sqlalchemy import create_engine

from db import connect_mysql

app = Flask(__name__)
CORS(app)

db_connection = connect_mysql()
cursor = db_connection.cursor()
# engine = create_engine('mysql+pymysql://root:@localhost:3306/KPI')





mergeFile = Blueprint('mergeFile', __name__)



@mergeFile.get('/')
@cross_origin()
def index():
    return 'Hello World!'


@mergeFile.get('/all_channel_id')
@cross_origin()
def all_channel_id():
    try:
        cursor.execute('''SELECT * FROM channelNewMaster''')

        return jsonify({'status': True, 'data': cursor.fetchall()})
    except Exception as e:
        return jsonify({'status': False, 'error': str(e)})


@mergeFile.post('/get_module_by_channel_id')
@cross_origin()
def get_module_by_channel_id():
    try:
        cursor.execute('''SELECT * FROM uploadModuleM WHERE channelId = %s''', (request.json['channelId'],))

        return jsonify({'status': True, 'data': cursor.fetchall()})
    except Exception as e:
        return jsonify({'status': False, 'error': str(e)})


@mergeFile.post('/get_columns_mapping')
@cross_origin()
def get_columns_mapping():
    try:
        module_id_name = request.json['moduleId']

        cursor.execute('''SELECT * FROM uploadModuleColMapping WHERE moduleId = %s AND tableColumnName IS NOT NULL''',
                       (module_id_name,))

        return jsonify({'status': True, 'data': cursor.fetchall()})
    except Exception as e:
        print(traceback.format_exc())
        return jsonify({'status': False, 'error': str(e)})


@mergeFile.post('/save_two_file_mapping')
@cross_origin()
def save_two_file_mapping():
    try:
        cursor.execute('DELETE FROM mappingTables')
        cursor.execute('DELETE FROM mappings')
        cursor.execute('DELETE FROM mapping_details')

        mapping_tables = [request.json['files1']['tableName'], request.json['files2']['tableName']]
        mapping_keys = '-'.join(mapping_tables)
        mapping_details = [
            [item['tableColumnName'] for item in request.json['columns1']],
            [item['tableColumnName'] for item in request.json['columns2']],
        ]

        cursor.execute('''INSERT INTO mappingTables (table_name) VALUES (%s), (%s)''',
                       (mapping_tables[0], mapping_tables[1]))

        cursor.execute('''INSERT INTO mappings (mapping_key) VALUES (%s)''', (mapping_keys,))

        cursor.execute('SELECT id FROM mappings WHERE mapping_key=%s', (mapping_keys,))
        mapping_id = cursor.fetchone()['id']

        if len(mapping_details[0]) == 1:
            cursor.execute(
                '''INSERT INTO mapping_details (mapping_id, source_column, target_column) VALUES (%s, %s, %s)''',
                (mapping_id, mapping_details[0][0], mapping_details[1][0]))
        else:
            for ind in range(len(mapping_details)):
                cursor.execute(
                    '''INSERT INTO mapping_details (mapping_id, source_column, target_column) VALUES (%s, %s, %s)''',
                    (mapping_id, mapping_details[0][ind], mapping_details[1][ind]))

        db_connection.commit()

        return jsonify({'status': True, 'data': 'Sucessfully Stored Mapping!'})
    except Exception as e:
        print(traceback.format_exc())
        return jsonify({'status': False, 'error': str(e)})


@mergeFile.post('/merge_two_file_tables')
@cross_origin()
def merge_two_file_tables():
    try:
        columns_to_keep = request.json['columnsToKeep']
        columns_to_keep = [item['tableColumnName'] for item in columns_to_keep]
        cursor.execute("SELECT table_name FROM mappingTables ORDER BY id")
        tables = [item['table_name'] for item in cursor.fetchall()]

        cursor.execute("SELECT mapping_key, id FROM mappings")
        mapping_keys = {item['mapping_key']: item['id'] for item in cursor.fetchall()}

        mapping_details = defaultdict(list)
        for mapping_key, mapping_id in mapping_keys.items():
            cursor.execute("SELECT source_column, target_column FROM mapping_details WHERE mapping_id = %s",
                           (mapping_id,))
            details = cursor.fetchall()
            mapping_details[mapping_key] = details

        for mapping_key, mapping_value in mapping_details.items():
            left = []
            right = []
            for maps in mapping_value:
                left.append(maps['source_column'])
                right.append(maps['target_column'])
            mapping_details[mapping_key] = [left, right]

        result = merge_two_tables(tables, mapping_details)

        df_filtered = list(result.values())[0][
            [col for col in columns_to_keep if col in list(result.values())[0].columns]].copy()

        df_filtered.to_sql(list(result.keys())[0], con=engine, index=False, if_exists='replace')

        cursor.execute(
            '''INSERT INTO uploadModuleM (moduleName, channelId, tableName, moduleDetails) VALUES (%s, %s, %s, %s)''',
            (list(result.keys())[0], request.json['channelId'], list(result.keys())[0], ''))

        cursor.execute('''SELECT id FROM uploadModuleM WHERE moduleName=%s''', (list(result.keys())[0],))
        module_id = cursor.fetchone()['id']

        [cursor.execute(
            '''INSERT INTO uploadModuleColMapping (moduleId, columnName, columnType, isBlankPossible, displayName, tableColumnName, activeFlag, monthYearFlag ) VALUES (%s, %s, %s, %s, %s, %s, %s, %s)''',
            (module_id, cols['columnName'], cols['columnType'], cols['isBlankPossible'], cols['displayName'],
             cols['tableColumnName'], cols['activeFlag'], cols['monthYearFlag']))
            for cols in
            request.json['columnsToKeep']]

        if len(list(result.keys())[0].split('-')) > 2:
            table_name = '-'.join(list(result.keys())[0].split('-')[:-1])
            cursor.execute(f"DROP TABLE IF EXISTS `{table_name}`")

            cursor.execute(f'''SELECT id FROM uploadModuleM WHERE moduleName="{table_name}"''')
            module_id = cursor.fetchone()['id']

            cursor.execute('''DELETE FROM uploadModuleM WHERE id=%s''', (module_id,))
            cursor.execute('''DELETE FROM uploadModuleColMapping WHERE moduleId=%s''', (module_id,))

        db_connection.commit()

        return jsonify({'status': True,
                        'merged_file_key': list(result.keys())[0]
                        })
    except Exception as e:
        print(traceback.format_exc())
        return jsonify({'status': False, 'error': str(e)})


def convert_int_columns_to_10_digits(df):
    for column in df.columns:
        if pd.api.types.is_integer_dtype(df[column]):
            df[column] = df[column].apply(lambda x: str(x).zfill(10))
    return df


def convert_string_columns_to_lower(df):
    for column in df.columns:
        if df[column].dtype == 'object':
            df[column] = df[column].str.lower()
    return df


def merge_two_tables(tables, mapping):
    final_df = {}

    temp = ''
    while len(tables):
        temp += f'{tables.pop(0)}-{tables.pop(0)}'

        final_df[temp] = pd.merge(
            convert_int_columns_to_10_digits(convert_string_columns_to_lower(
                pd.read_sql(f"""SELECT * FROM {temp.split('-')[0]}""", con=engine))),
            convert_int_columns_to_10_digits(convert_string_columns_to_lower(
                pd.read_sql(f"""SELECT * FROM {temp.split('-')[1]}""", con=engine))),
            left_on=mapping[temp][0],
            right_on=mapping[temp][1],
            how='left'
        )

    return final_df


# if __name__ == '__main__':
#     app.run()
